import numpy as np
import matplotlib
import matplotlib.gridspec as gridspec
matplotlib.use('Agg')  # so figs just print to file. Needs to come before pyplot
import matplotlib.pyplot as plt
import scipy.stats
from sklearn import metrics
import pickle
import os
import sys
import src.ml_load as ml_load
import warnings

# make plot to show Rsq for different generalization experiments

# plotting defaults
matplotlib.rcParams['agg.path.chunksize'] = 10000
matplotlib.rcParams.update({'font.size': 8})

n_test_exs=None
rain_only=False 
no_cos=False 
no_ps=False 
use_rh=False
conv_cond=False

expt1 = 'abs1.0_norf_ras' 
expt2 = 'abs1.4_norf_ras' 

est_str1 = 'abs1.0_norf_ras_X-NoScaler_Y-SimpleY_Ntrnex700000_RF_regress_NumTrees10_MinSamples10_MaxFeaturesThird_minlev0.08' 
est_str2 = 'abs1.4_norf_ras_X-NoScaler_Y-SimpleY_Ntrnex700000_RF_regress_NumTrees10_MinSamples10_MaxFeaturesThird_minlev0.08' 
est_str2_xtr = 'abs1.4_norf_ras_X-NoScaler_Y-SimpleY_Ntrnex700000_RF_regress_NumTrees10_MinSamples10_MaxFeaturesThird_minlev0.08_xtropics25' 

datadir1, trainfile1, testfile1, pp_str1 = ml_load.GetDataPath(conv_cond, expt1)
datadir2, trainfile2, testfile2, pp_str2 = ml_load.GetDataPath(conv_cond, expt2)


# Open the estimator and the preprocessing scheme
est_eval1, _, _, x_ppi1, y_ppi1, x_pp1, y_pp1, lat, lev, dlev = \
    pickle.load(open('/net/esker/archive1/pog/mldata/regressors/' + est_str1 + '.pkl', 'rb'))
est_eval2, _, _, x_ppi2, y_ppi2, x_pp2, y_pp2, lat, lev, dlev = \
    pickle.load(open('/net/esker/archive1/pog/mldata/regressors/' + est_str2 + '.pkl', 'rb'))
est_eval2_xtr, _, _, x_ppi2_xtr, y_ppi2_xtr, x_pp2_xtr, y_pp2_xtr, lat, lev, dlev = \
    pickle.load(open('/net/esker/archive1/pog/mldata/regressors/' + est_str2_xtr + '.pkl', 'rb'))


# Load data at each level and latitude

# Warm climate, trained on control climate
_,_,Tvar_warm_control,_,_,_,_,_,_,_,Rsq_T_warm_control,_  = \
      ml_load.stats_by_latlev(x_ppi1, y_ppi1, x_pp1, y_pp1, est_eval1, lat,
                             lev, testfile2, n_test_exs, rain_only, 
                             no_cos, no_ps, use_rh)

# Cold climate, trained on warm climate
_,_,Tvar_control_warm,_,_,_,_,_,_,_,Rsq_T_control_warm,_  = \
      ml_load.stats_by_latlev(x_ppi2, y_ppi2, x_pp2, y_pp2, est_eval2, lat,
                             lev, testfile1, n_test_exs, rain_only, 
                             no_cos, no_ps, use_rh)

# Cold climate, trained on warm climate xtropics
_,_,Tvar_control_warm_xtr,_,_,_,_,_,_,_,Rsq_T_control_warm_xtr,_  = \
      ml_load.stats_by_latlev(x_ppi2_xtr, y_ppi2_xtr, x_pp2, y_pp2_xtr, est_eval2_xtr, lat,
                             lev, testfile1, n_test_exs, rain_only, 
                             no_cos, no_ps, use_rh)

# Warm climate, trained on warm climate xtropics
_,_,Tvar_warm_warm_xtr,_,_,_,_,_,_,_,Rsq_T_warm_warm_xtr,_  = \
      ml_load.stats_by_latlev(x_ppi2_xtr, y_ppi2_xtr, x_pp2, y_pp2_xtr, est_eval2_xtr, lat,
                             lev, testfile2, n_test_exs, rain_only, 
                             no_cos, no_ps, use_rh)

# mask Rsq where variance is very low
mask_too_low = 0.01
too_low = np.nonzero(Tvar_warm_control<mask_too_low*np.mean(Tvar_warm_control))
Rsq_T_warm_control[too_low]=np.nan

too_low = np.nonzero(Tvar_control_warm<mask_too_low*np.mean(Tvar_control_warm))
Rsq_T_control_warm[too_low]=np.nan

too_low = np.nonzero(Tvar_control_warm_xtr<mask_too_low*np.mean(Tvar_control_warm_xtr))
Rsq_T_control_warm_xtr[too_low]=np.nan

too_low = np.nonzero(Tvar_warm_warm_xtr<mask_too_low*np.mean(Tvar_warm_warm_xtr))
Rsq_T_warm_warm_xtr[too_low]=np.nan

# make plot
f, axes = plt.subplots(2, 2, figsize=(6,3.5))
ax1 = axes[0, 0]
ax2 = axes[0, 1]
ax3 = axes[1, 0]
ax4 = axes[1, 1]

# contour levels
levels = np.linspace(0.2,1,9) 

x = [-60,-30,0,30,60]
#x = [-75,-50,-25,0,25,50,75]
y = [0.2,0.4,0.6,0.8,1.0]
cax1 = ax1.contourf(lat, lev, Rsq_T_warm_control, levels, extend='min')
ax1.set_ylim(1, min(lev))
ax1.set_ylabel(r'Vertical level ($\sigma$)')
#ax1.xaxis.set_ticklabels([])
ax1.xaxis.set_ticks(x)
ax1.yaxis.set_ticks(y)
ax1.set_title('(a) Warm climate, trained control', fontsize=8)

cax2 = ax2.contourf(lat, lev, Rsq_T_control_warm, levels, extend='min')
ax2.set_ylim(1, min(lev))
#ax2.xaxis.set_ticklabels([])
#ax2.yaxis.set_ticklabels([])
ax2.xaxis.set_ticks(x)
ax2.yaxis.set_ticks(y)
ax2.set_title('(b) Control climate, trained warm', fontsize=8)

cax3 = ax3.contourf(lat, lev, Rsq_T_warm_warm_xtr, levels, extend='min')
ax3.set_ylim(1, min(lev))
ax3.set_xlabel('Latitude (degrees)')
ax3.set_ylabel(r'Vertical level ($\sigma$)')
ax3.xaxis.set_ticks(x)
ax3.yaxis.set_ticks(y)
ax3.set_title('(c) Warm climate, trained warm xtropics', fontsize=8)

cax4 = ax4.contourf(lat, lev, Rsq_T_control_warm_xtr, levels, extend='min')
ax4.set_ylim(1, min(lev))
ax4.set_xlabel('Latitude (degrees)')
ax4.xaxis.set_ticks(x)
ax4.yaxis.set_ticks(y)
#ax4.yaxis.set_ticklabels([])
ax4.set_title('(d) Control climate, trained warm xtropics', fontsize=8)

# wspace controls width between plots, hspace controls height between plots
f.subplots_adjust(right=0.9, wspace=0.2, hspace=0.45)

# add axes at [left, bottom, width, height]
cbar_ax = f.add_axes([0.93, 0.2, 0.02, 0.6])
f.colorbar(cax2, cax=cbar_ax, ticks=np.arange(0.2,1.1,0.1))

f.savefig('generalization_xtropics25.eps', bbox_inches='tight')
plt.close()


